<?php
/**
 * Plugin Name:         RS Duplicate Post
 * Description:         Simple plugin to duplicate posts.
 * Plugin URI:          https://plugins.seindal.dk/plugins/rs-duplicate-post/
 * Update URI:          https://plugins.seindal.dk
 * Author:              René Seindal
 * Author URI:          https://plugins.seindal.dk/
 * Donate link:         https://mypos.com/@historywalks
 * Requires Plugins:    rs-base-plugin
 * License:             GPL v2 or later
 * License URI:         https://www.gnu.org/licenses/gpl-2.0.html
 * Text Domain:         rs-duplicate-post
 * Domain Path:         /languages
 * Requires PHP:        7.4
 * Requires at least:   5.0
 * Version:             1.0.1
 **/

namespace ReneSeindal;

if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

require_once( ABSPATH . 'wp-admin/includes/taxonomy.php' );

add_action( 'plugins_loaded', function() {
    class DuplicatePostPlugin extends PluginBase {
        protected $plugin_file = __FILE__;

        /*************************************************************
         *
         * Accept all post types for now
         *
         *************************************************************/

        protected function post_type_ok( $post ) {
            return true;
        }


        /*************************************************************
         *
         * Add links to post row
         *
         *************************************************************/

        protected $post_row_actions_callback_arguments = 2;

        function do_post_row_actions_filter_duplicate_post( $actions, $post ) {
            if ( $this->post_type_ok( $post ) ) {
                $url = $this->build_admin_url( 'admin-post.php', [
                    'action' => 'duplicate_post',
                    'post' => $post->ID,
                ] );

                $link = sprintf( '<a href="%1$s">%2$s</a>',
                                 esc_url( $url ),
                                 esc_html( __( 'Duplicate', 'rs-duplicate-post' ) )
                );
                $actions[ 'duplicate_post' ] = $link;
            }

            return $actions;
        }

        protected $page_row_actions_callback_arguments = 2;

        function do_page_row_actions_filter_duplicate_post( $actions, $post ) {
            return $this->do_post_row_actions_filter_duplicate_post( $actions, $post );
        }

        /*************************************************************
         *
         * Duplicate a post and redirect to editor
         *
         *************************************************************/

        function do_admin_post_duplicate_post_action() {
            if ( ! isset( $_GET['post'] ) ) return;
            if ( ! is_numeric( $_GET['post'] ) ) return;

            $post_id = intval( $_GET['post'] );

            if ( $post_id <= 0 ) return;

            $post = get_post( $post_id );
            if ( ! $post ) return;

            // Basic changes
            $data = get_object_vars( $post );

            $data['ID'] = 0;
            $data['post_status'] = 'draft';
            $data['post_title'] = $data['post_title'] . ' ' . __( '(copy)', 'rs-duplicate-post' );

            $delete_fields = [ 'guid', 'post_name', 'post_date', 'post_date_gmt', 'post_modified', 'post_modified_gmt' ];
            foreach ( $delete_fields as $field ) {
                unset( $data[ $field ] );
            }

            // Find taxonomies for this post
            $taxonomies = array_map(
                fn($t) => $t->name,
                array_filter(
                    get_taxonomies( [ 'show_ui' => true ], 'objects' ),
                    fn($t) => in_array( $post->post_type, $t->object_type )
                )
            );

            foreach ( $taxonomies as $taxonomy ) {
                $terms = wp_get_post_terms( $post_id, $taxonomy, [ 'fields' => 'ids' ] );
                if ( ! empty( $terms ) ) {
                    $data['tax_input'][$taxonomy] = $terms;
                }
            }


            $meta = get_post_meta( $post_id );
            foreach ( $meta as $key => $value ) {
                if ( '_thumbnail_id' == $key )
                    $value = $value[0];
                elseif ( str_starts_with( $key, '_' ) )
                    $value = NULL;
                else
                    $value = $value[0];

                if ( $value )
                    $data['meta_input'][$key] = $value;
            }

            $id = wp_insert_post( $data, true );

            if( is_wp_error( $id ) ) {
                echo $id->get_error_message();
                return;
            }

            $url = $this->build_admin_url( 'post.php', [ 'post' => $id, 'action' => 'edit' ] );
            wp_redirect( $url );
            exit;
        }
    }

    new DuplicatePostPlugin();
} );
